/**
 * Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
 * 
 * You should have find a copy of NFSChina License. If not, please contact us 
 * by email <os_support@nfschina.com>  
 * 
**/

#ifndef LIBFDDISKINFO_H
#define LIBFDDISKINFO_H

#include <stdlib.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @struct FDKDiskInfo
 * @brief 表示硬盘设备信息
 *
 * 该结构体包含硬盘设备的名称、型号、容量、序列号等信息。
 */
typedef struct {
    unsigned char *device_name;   /**< 硬盘设备名称 */
    unsigned char *model;          /**< 硬盘型号 */
    unsigned char *serial_number;  /**< 硬盘序列号 */
    unsigned char *size;            /**< 硬盘大小 */
    int rotational;                 /**< 是否旋转（1 = HDD，0 = SSD） */
    unsigned char *firmware_version; /**< 固件版本号，动态分配的字符串 */
    int partition_count;    /**< 分区数 */
} FDKDiskInfo;

/**
 * @struct FDKDiskFirmwareInfo
 * @brief 硬盘固件信息
 *
 * 该结构体包含硬盘设备的名称、固件版本号以及固件更新支持信息。
 */
typedef struct {
    unsigned char *device_name;  /**< 磁盘设备名称，例如 "/dev/sda" */
    unsigned char *firmware_version;  /**< 固件版本号（例如 "V1.0.3"） */
    int update_enabled;               /**< 是否支持固件更新（1 = 支持更新，0 = 不支持更新） */
}FDKDiskFirmwareInfo;

/**
 * 磁盘分区信息结构体。
 *
 * 该结构体用于存储关于磁盘分区的详细信息，包括设备名称、分区大小、已用空间、文件系统类型等。
 */
typedef struct {
    unsigned char *disk_device_name;  /**< 磁盘设备名称，例如 "/dev/sda"。该字段表示磁盘的设备路径。 */
    unsigned char *partition_name;    /**< 分区设备名称，例如 "/dev/sda1"。该字段表示磁盘分区的设备路径。 */
    unsigned long long size;          /**< 分区大小，单位为字节（Bytes）。该字段存储分区的总大小。 */
    unsigned long long used_space;    /**< 已使用空间，单位为字节（Bytes）。该字段表示分区中已被使用的空间大小。 */
    unsigned long long free_space;    /**< 剩余空间，单位为字节（Bytes）。该字段表示分区中未被使用的可用空间大小。 */
    unsigned char *filesystem_type;   /**< 文件系统类型（例如 "ext4", "NTFS"）。该字段存储分区所使用的文件系统类型。 */
    unsigned char *mount_point;       /**< 挂载点路径（例如 "/mnt/data"）。该字段表示分区在系统中的挂载点路径。 */
    int is_bootable;                  /**< 是否为启动分区，1为是，0为否。该字段指示该分区是否为可引导分区。 */
    int is_encrypted;                 /**< 是否为加密分区，1为是，0为否。该字段指示该分区是否启用了加密。 */
} FDKDiskPartitionInfo;

/**
 * 获取计算机中磁盘名称列表。
 *
 * 该接口用于获取系统中所有硬盘的设备名称（例如 "/dev/sda"），并将其存放在动态分配的磁盘名称列表中。
 * 内存由函数内部分配，调用者在使用完后需调用相应的释放函数以避免内存泄漏。
 *
 * @param[out] count 存放磁盘数量的变量，调用者分配内存，用于返回硬盘数量。
 *                   调用者必须提供有效的指针地址。返回的磁盘数量将存放在该指针所指向的内存中。
 * @param[out] errcode 错误码输出参数。如果成功，返回 0；失败时返回具体错误码。
 * @return 返回所有硬盘的设备名称（例如 "/dev/sda"）的列表。
 *         如果成功，返回指向磁盘名称数组的指针；如果失败，返回 `NULL`。
 * @note 调用者应确保传入的 `count` 和 `errcode` 地址有效且可以访问。
 * @warning 调用者在使用完返回的磁盘名称列表后，必须调用 `fdk_hwinfo_free_disks` 函数释放内存，
 *          以避免内存泄漏。
 */
unsigned char **fdk_diskinfo_get_disk_list(size_t *count, int *errcode);

/**
 * 获取所有硬盘的详细信息。
 *
 * 该函数通过系统命令获取当前硬盘的相关信息，并将其存储到一个动态分配的硬盘信息数组中。
 * 该函数返回系统中所有硬盘的信息，内存由函数内部分配，调用者在使用完返回的数据后需调用
 * `fdk_diskinfo_free_all_disk_info` 释放内存。
 * @param[out] count 指向存放硬盘信息数量的指针，由调用者分配内存。
 * @param[out] errcode 错误码，成功时为 0，失败时返回错误码。
 * @return 返回指向 `FDKDiskInfo` 结构体数组的指针，成功时返回数组，失败时返回 `NULL`。
 * @warning 调用者需确保传入的 `count` 指针有效且可以访问。
 * @note 提供当前系统所有硬盘的详细信息，包括设备名称、容量、制造商、型号、序列号等。
 *       该函数由API内部分配内存，调用者无需自行分配内存。调用完成后，调用者应释放返回的数据内存。
 */
FDKDiskInfo **fdk_diskinfo_get_all_disk_info(size_t *count, int *errcode);

/**
 * 释放由 fdk_diskinfo_get_all_disk_info 返回的硬盘信息数组及其内存
 * @param[in] disk_info 指向硬盘信息数组的指针
 * @param[in] count 硬盘信息数量
 * @note 调用此接口后，传入的 disk_info 数组及其中的每个元素将不再有效。
 */
void fdk_diskinfo_free_all_disk_info(FDKDiskInfo **disk_info, size_t count);

/**
 * 获取指定硬盘的详细信息。
 *
 * 该函数通过系统命令获取指定硬盘（如 `/dev/sda`）的相关信息，并将其存储在动态分配的 `FDKDiskInfo` 结构体中。
 * 返回的结构体内存由函数内部分配，调用者在使用完后需通过 `fdk_diskinfo_free_disk_info` 函数释放内存。
 * @param[in] disk_device 指定要获取信息的磁盘设备名称（例如 `/dev/sda`）。
 * @param[out] errcode 错误码输出参数。成功时返回 0，失败时返回具体的错误码。
 * @return 返回指向 `FDKDiskInfo` 结构体的指针，成功时包含该磁盘的详细信息；失败时返回 `NULL`。
 * @warning 调用者应确保传入的 `disk_device` 参数有效且可以访问，避免无效路径。
 * @note 该函数通过系统命令获取硬盘信息，返回的信息包括设备名称、容量、制造商、型号、序列号等。
 *       调用者在使用完返回的数据后，必须释放内存，以避免内存泄漏。
 * @see fdk_diskinfo_free_disk_info
 */
FDKDiskInfo *fdk_diskinfo_get_disk_info(const unsigned char *disk_device, int *errcode);

/**
 * 释放指定硬盘信息内存。
 *
 * 该函数用于释放通过 `fdk_diskinfo_get_disk_info` 函数获取的硬盘信息结构体 `FDKDiskInfo` 所分配的动态内存。
 * 确保在使用完硬盘信息后，调用此函数以避免内存泄漏。
 *
 * @param[in] disk_info 指向 `FDKDiskInfo` 结构体的指针，表示硬盘的详细信息。
 *                      该结构体由 `fdk_diskinfo_get_disk_info` 函数动态分配。
 *                      调用此函数后，`disk_info` 指向的内存将被释放，调用者不能继续访问该结构体。
 * @return void 无返回值。
 * @note 调用此函数后，`disk_info` 不再有效。调用者必须确保仅释放一次该内存，以避免双重释放。
 * @warning 请勿重复调用 `fdk_diskinfo_free_disk_info` 释放同一内存块，确保传入有效的指针。
 * @see fdk_diskinfo_get_disk_info
 */
void fdk_diskinfo_free_disk_info(FDKDiskInfo *disk_info);

/**
 * 获取指定磁盘的固件信息。
 *
 * 该接口通过系统命令或读取硬件信息接口获取指定磁盘的固件版本，并将其存储到动态分配的 `FDKDiskFirmwareInfo` 结构体中。
 * 调用者在使用完返回的固件信息后需要释放内存。
 *
 * @param[in] disk_device 磁盘设备名称，例如 "/dev/sda"。该参数用于指定获取固件信息的磁盘设备。
 *                        确保传入的设备名称正确且设备存在。
 * @param[out] errcode 错误码输出参数。如果成功返回 0，失败时返回具体的错误码。
 *                     调用者必须提供有效的指针地址。
 * @return 返回动态分配的 `FDKDiskFirmwareInfo` 结构体指针，成功时返回指向结构体的指针，失败时返回 `NULL`。
 * @warning 调用者必须确保在使用完返回的固件信息后，调用 `fdk_diskinfo_free_firmware_info` 释放内存，以避免内存泄漏。
 * @see fdk_diskinfo_free_firmware_info
 */
FDKDiskFirmwareInfo *fdk_diskinfo_get_firmware_info(const unsigned char *disk_device, int *errcode);

/**
 * 释放指定磁盘信息内存。
 *
 * 该函数用于释放通过 `fdk_diskinfo_get_firmware_info` 获取的 `FDKDiskFirmwareInfo` 结构体的内存。
 * 调用该函数后，传入的指针所指向的内存将被释放，调用者不再可以访问该结构体。
 *
 * @param[in] firmware_info 指向 `FDKDiskFirmwareInfo` 结构体的指针，该结构体保存了硬盘设备的固件信息。
 *                          该指针必须由 `fdk_diskinfo_get_firmware_info` 函数返回。
 * @return void 无返回值。
 * @note 确保传入的 `firmware_info` 为有效指针，且已调用 `fdk_diskinfo_get_firmware_info` 获取信息。
 * @warning 不要重复释放同一块内存，确保只调用一次该释放函数。
 */
void fdk_diskinfo_free_firmware_info(FDKDiskFirmwareInfo *firmware_info);


/**
 * 获取指定磁盘的分区信息。
 *
 * 该接口用于获取指定磁盘的所有分区信息，并将其存储到由调用者提供的分区信息列表中。
 * 内存由函数内部分配，调用者在使用完返回的数据后需调用相应的释放函数以避免内存泄漏。
 *
 * @param[in] disk_device 磁盘设备名称，例如 "/dev/sda"。
 * @param[out] count 存放分区数量的变量，调用者分配内存，用于返回分区数量。
 *                   调用者必须提供有效的指针地址。
 * @param[out] errcode 错误码输出参数。如果成功，返回 0；失败时返回具体错误码。
 * @return FDKDiskPartitionInfo** 成功返回包含分区信息的数组，失败时返回 NULL。
 * @note 调用者应确保传入的 `disk_device`、`partition_info` 和 `count` 地址有效且可以访问。
 * @warning 调用者在使用完返回的分区信息列表后，必须释放内存，以避免内存泄漏。
 * @see fdk_diskinfo_free_partition_info
 */
FDKDiskPartitionInfo **fdk_diskinfo_get_disk_partition_info(const unsigned char *disk_device, int *count, int *errcode);

/**
 * 释放磁盘分区信息的内存。
 *
 * 该函数用于释放由 `fdk_diskinfo_get_disk_partition_info` 或其他相关函数分配的分区信息数组。
 * 该数组包含多个 `FDKDiskPartitionInfo` 结构体，每个结构体中可能包含动态分配的字符串（如设备名称、文件系统类型等）。
 * 调用此函数后，内存将被正确释放，以避免内存泄漏。
 * @param[in] partition_info_array 指向分区信息数组的指针。每个元素是 `FDKDiskPartitionInfo` 结构体的指针。
 * @param[in] count 分区信息数组中的元素个数。
 * @note 确保在调用此函数之前，分区信息数组已经使用完毕，并且不再需要访问。
 */
void fdk_diskinfo_free_partition_info(FDKDiskPartitionInfo **partition_info_array, int count);


#ifdef __cplusplus
}
#endif


#endif // LIBFDDISKINFO_H
