/**
 * Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
 * 
 * You should have find a copy of NFSChina License. If not, please contact us 
 * by email <os_support@nfschina.com>  
 * 
**/

#ifndef LIBFDHWINFO_H
#define LIBFDHWINFO_H

#include <stdlib.h>

#ifdef __cplusplus
extern "C" {
#endif


/**
 * @struct FDKBiosInfo
 * @brief 表示 BIOS 信息
 * 
 * 该结构体包含 BIOS 的制造商、版本和日期等信息。
 */
typedef struct {
    unsigned char *vendor;    /**< BIOS 制造商 */
    unsigned char *version;   /**< BIOS 版本 */
    unsigned char *date;      /**< BIOS 日期 */
} FDKBiosInfo;


/**
 * @struct FDKMotherboardInfo
 * @brief 表示主板信息
 * 
 * 该结构体包含主板的制造商、名称、版本、芯片组、SMBIOS 版本和序列号等信息。
 */
typedef struct {
    unsigned char *vendor;        /**< 主板制造商 */
    unsigned char *version;       /**< 主板版本 */
    unsigned char *chipset;       /**< 主板芯片组 */
    unsigned char *smbios_version; /**< SMBIOS 版本 */
    unsigned char *serial;        /**< 主板序列号 */
} FDKMotherboardInfo;


/**
 * @struct FDKCpuInfo
 * @brief 表示 CPU 信息
 * 
 * 该结构体包含 CPU 的制造商、型号、架构、核心数、线程数、缓存信息和虚拟化支持等信息。
 */
typedef struct {
    unsigned char *manufacturer;  /**< CPU 制造商 */
    unsigned char *model;        /**< CPU 型号 */
    unsigned char *architecture; /**< CPU 架构 */
    unsigned int cores;          /**< 核心数 */
    unsigned int threads;        /**< 线程数 */
    unsigned char *cache_l1_data; /**< 一级缓存 (数据) */
    unsigned char *cache_l1_instruction; /**< 一级缓存 (指令) */
    unsigned char *cache_l2;      /**< 二级缓存 */
    unsigned char *cache_l3;      /**< 三级缓存 */
    unsigned char *virtualization; /**< 虚拟化支持 */
} FDKCpuInfo;


/**
 * @struct FDKMemoryInfo
 * @brief 表示内存信息
 * 
 * 该结构体包含内存的制造商、容量、速度、位宽、序列号等信息。
 */
typedef struct {
    unsigned char *manufacturer; /**< 内存制造商 */
    unsigned char *capacity;     /**< 内存容量 */
    unsigned char *speed;        /**< 内存速度 */
    unsigned char *width;        /**< 内存位宽 */
    unsigned char *serial;       /**< 内存序列号 */
} FDKMemoryInfo;

/**
 * @struct FDKGPUInfo
 * @brief 表示显卡信息
 * 
 * 该结构体包含显卡的制造商、名称、型号、类型、显存、接口、驱动和时钟频率等信息。
 */
typedef struct {
    unsigned char *manufacturer;  /**< 显卡制造商 */
    unsigned char *name;          /**< 显卡名称 */
    unsigned char *model;         /**< 显卡型号 */
    unsigned char *type;          /**< 显卡类型 */
    unsigned char *interface;     /**< 接口 */
    unsigned char *driver;        /**< 驱动 */
    unsigned char *clock_speed;   /**< 时钟频率 */
} FDKGPUInfo;

/**
 * @struct FDKNetworkCardInfo
 * @brief 表示网络接口卡（NIC）的详细信息
 *
 * 该结构体用于存储系统中每个网络接口卡的相关信息，包括名称、制造商、类型、驱动程序、MAC 地址等。
 */
typedef struct {
    unsigned char *name;
    unsigned char *manufacturer;
    unsigned char *type;
    unsigned char *version;
    unsigned char *capabilities;
    unsigned char *driver;
    int enabled;
    unsigned char *mac_address;
} FDKNetworkCardInfo;

/**
 * 显示器信息结构体。
 *
 * 该结构体用于存储显示器的相关信息，包括厂商、型号、尺寸、生产日期、分辨率和刷新率等。
 * 显示器的信息将在通过 `getAllDisplaysInfo` 函数获取时被填充到该结构体中。
 */
typedef struct FDKDisplayInfo {
    unsigned char *serial;          /**< 显示器序列号，动态分配的字符串 */
    unsigned char *manufacturer;    /**< 显示器厂商，动态分配的字符串 */
    unsigned char *model;           /**< 显示器型号，动态分配的字符串 */
    unsigned char *size;            /**< 显示器尺寸，单位为英寸，动态分配的字符串 */
    unsigned char *production_date; /**< 显示器生产日期，动态分配的字符串 */
    int resolution_width;           /**< 显示器分辨率宽度，单位为像素 */
    int resolution_height;          /**< 显示器分辨率高度，单位为像素 */
    unsigned char *refresh_rate;            /**< 显示器支持的刷新率 */
} FDKDisplayInfo;

/**
 * 蓝牙设备信息结构体。
 *
 * 该结构体用于存储蓝牙设备的相关信息，包括设备名称、驱动程序、速度、物理地址、连接模式等。
 * 蓝牙设备的信息将在通过 `fdk_hwinfo_get_bluetooth_adapter_info` 函数获取时被填充到该结构体中。
 */
typedef struct FDKBluetoothDeviceInfo {
    unsigned char *id;              /**< 蓝牙设备ID，动态分配的字符串 */
    unsigned char *manufacturer;    /**< 蓝牙设备厂商，动态分配的字符串 */
    unsigned char *name;           /**< 蓝牙设备名称，动态分配的字符串 */
    unsigned char *version;        /**< 蓝牙设备版本，动态分配的字符串 */
    unsigned char *driver;         /**< 蓝牙设备驱动，动态分配的字符串 */
    unsigned char *speed;          /**< 蓝牙设备速度，动态分配的字符串 */
    unsigned char *physical_address; /**< 蓝牙设备的物理地址，动态分配的字符串 */
    unsigned char *connection_mode; /**< 蓝牙设备连接模式，动态分配的字符串 */
} FDKBluetoothDeviceInfo;

/**
 * PCI设备信息结构体。
 *
 * 该结构体用于存储PCI设备的相关信息，包括设备ID、设备名称等。
 * PCI设备的信息将在通过 `fdk_hwinfo_get_pci_device_info` 函数获取时被填充到该结构体中。
 */
typedef struct FDKPciDeviceInfo {
    unsigned char *device_id;      /**< PCI设备ID，动态分配的字符串 */
    unsigned char *device_type;      /**< PCI设备类型，动态分配的字符串 */
    unsigned char *device_name;    /**< PCI设备名称，动态分配的字符串 */
    unsigned char *subsystem;      /**< PCI设备子系统，动态分配的字符串 */
    unsigned char *flags;          /**< PCI设备标志，动态分配的字符串 */
    unsigned char **memory;         /**< PCI设备内存，动态分配的字符串 */
    int memory_count;               /**< PCI设备内存个数 */
    unsigned char *kernel_driver;  /**< PCI设备内核驱动，动态分配的字符串 */
    unsigned char *kernel_modules; /**< PCI设备内核模块，动态分配的字符串 */
} FDKPciDeviceInfo;

/**
 * 获取 BIOS 信息，包含制造商、版本号和发布日期
 * @param[out] bios_info 指向 `FDKBiosInfo` 结构体指针的指针，成功时返回 BIOS 信息，否则返回 `NULL`。
 * @return 成功返回 0，失败返回错误码。
 * @warning 调用者需确保在使用完返回的 `FDKBiosInfo` 结构体后进行释放，避免内存泄漏。
 * @note 提供当前系统的 BIOS 制造商、版本和发布日期等信息，具体内容取决于系统的硬件信息。
 */
int fdk_hwinfo_get_bios_info(FDKBiosInfo **bios_info);

/**
 * 释放由 fdk_hwinfo_get_bios_info 返回的内存
 * @param[in] bios_info 指向需要释放的 BIOS 信息结构体指针
 * @note 调用此接口后，传入的 `bios_info` 指针将不再有效。
 * @warning 请确保在调用此接口之前，已经不再使用传入的 `bios_info`。
 */
void fdk_hwinfo_free_bios_info(FDKBiosInfo *bios_info);

/**
 * 获取主板信息
 * @param[out] mb_info 指向 `FDKMotherboardInfo` 结构体指针的指针，成功时返回主板信息，否则返回 `NULL`。
 * @return 成功返回 0，失败返回错误码。
 * @warning 调用者需确保在使用完返回的 `FDKMotherboardInfo` 结构体后，释放其内存，避免内存泄漏。
 * @note 提供当前系统主板的详细信息，包括芯片组和 SMBIOS 版本等。
 */
int fdk_hwinfo_get_motherboard_info(FDKMotherboardInfo **mb_info);

/**
 * 释放由 `fdk_hwinfo_get_motherboard_info` 返回的内存
 * @param[in] mb_info 指向需要释放的 `FDKMotherboardInfo` 结构体指针。
 * @note 调用此接口后，传入的 `mb_info` 指针将不再有效。
 */
void fdk_hwinfo_free_motherboard_info(FDKMotherboardInfo *mb_info);

/**
 * 获取 CPU 信息
 * @param[out] count 存放 CPU 信息数量的指针，由调用者分配内存。
 * @param[out] errcode 错误码，0 表示成功，其他值表示失败的具体原因。
 * @return 返回指向 `FDKCpuInfo` 结构体数组的指针，成功时返回数组，失败时返回 `NULL`。
 * @warning 调用者需确保传入的 `count` 指针有效且可以访问。
 * @note 提供详细的 CPU 硬件规格，包括缓存信息和虚拟化支持。
 */
FDKCpuInfo **fdk_hwinfo_get_cpu_info(size_t *count, int *errcode);

/**
 * 释放由 `fdk_hwinfo_get_cpu_info` 返回的CPU信息数组及其内存
 * @param[in] cpu_info 指向 `FDKCpuInfo` 结构体数组的指针
 * @param[in] count CPU信息数量
 * @note 调用此接口后，传入的 `cpu_info` 数组及其中的每个元素将不再有效。
 */
void fdk_hwinfo_free_cpu_info(FDKCpuInfo **cpu_info, size_t count);

/**
 * 获取内存信息，包含制造商、容量、速度、位宽、序列号等
 *
 * 该接口通过 `dmidecode` 命令获取内存硬件的详细信息，包括容量、速度、序列号等。内存信息会存储在返回的 `FDKMemoryInfo` 结构体数组中。
 * 内存由函数内部分配，调用者不需要自行分配内存。
 *
 * @param[out] count 指向存放内存信息数量的指针，由调用者分配内存。
 * @param[out] errcode 错误码，若成功返回 0，失败时返回相应的错误码。
 * @return 返回指向 `FDKMemoryInfo` 结构体数组的指针，成功时返回数组，失败时返回 `NULL`。
 * @warning 调用者需确保传入的 `count` 指针有效且可以访问。
 * @note 提供有关系统内存的详细信息，包括容量、速度和序列号等。
 * @see 无
 */
FDKMemoryInfo **fdk_hwinfo_get_memory_info(size_t *count, int *errcode);

/**
 * 释放由 `fdk_hwinfo_get_memory_info` 返回的内存信息数组及其内存
 * @param[in] memory_info 指向内存信息数组的指针
 * @param[in] count 内存信息数量
 * @note 调用此接口后，传入的 `memory_info` 数组及其中的每个元素将不再有效。
 */
void fdk_hwinfo_free_memory_info(FDKMemoryInfo **memory_info, size_t count);

/**
 * 获取显卡信息，包含显卡数量，每块显卡的制造商、名称、型号、类型、显存、接口、驱动、时钟频率等
 *
 * @param[out] count 指向存放显卡信息数量的指针，由调用者分配内存。
 * @param[out] errcode 错误代码，调用时传入指针以接收错误信息。
 * @return 返回指向 `FDKGPUInfo` 结构体数组的指针，成功时返回数组，失败时返回 `NULL`。
 * @warning 调用者需确保传入的 `count` 和 `errcode` 指针有效且可以访问。
 * @note 提供当前系统显卡的详细信息，包括显存大小和接口类型等。
 *       该函数由API内部分配内存，调用者不需要自行分配内存。
 */
FDKGPUInfo **fdk_hwinfo_get_gpu_info(size_t *count, int *errcode);

/**
 * 释放由 `fdk_hwinfo_get_gpu_info` 返回的显卡信息数组及其内存
 * @param[in] gpu_info 指向显卡信息数组的指针
 * @param[in] count 显卡信息数量
 * @note 调用此接口后，传入的 `gpu_info` 数组及其中的每个元素将不再有效。
 */
void fdk_hwinfo_free_gpu_info(FDKGPUInfo **gpu_info, size_t count);

/**
 * 获取有线网卡信息，包含网卡的名称、制造商、版本、驱动、启用状态、MAC地址等
 *
 * @param[out] count 指向存放网卡信息数量的指针，由调用者分配内存。
 * @param[out] errcode 错误代码，调用时传入指针以接收错误信息。
 * @return 返回指向 `FDKNetworkCardInfo` 结构体数组的指针，成功时返回数组，失败时返回 `NULL`。
 * @warning 调用者需确保传入的 `count` 和 `errcode` 指针有效且可以访问。
 * @note 提供当前系统中所有网卡的详细信息，包括MAC地址和启用状态等。
 *       该函数由API内部分配内存，调用者不需要自行分配内存。
 */
FDKNetworkCardInfo **fdk_hwinfo_get_network_card_info(size_t *count, int *errcode);

#if 0
/**
 * 获取WiFi设备信息
 *
 * @param[out] count 指向存放网卡信息数量的指针，由调用者分配内存。
 * @param[out] errcode 错误代码，调用时传入指针以接收错误信息。
 * @return 返回指向 `FDKNetworkCardInfo` 结构体数组的指针，成功时返回数组，失败时返回 `NULL`。
 * @warning 调用者需确保传入的 `count` 和 `errcode` 指针有效且可以访问。
 * @note 提供当前系统中所有网卡的详细信息，包括MAC地址和启用状态等。
 *       该函数由API内部分配内存，调用者不需要自行分配内存。
 */
FDKNetworkCardInfo **fdk_hwinfo_get_wlan_card_info(size_t *count, int *errcode);
#endif
/**
 * 释放由 `fdk_hwinfo_get_network_card_info` 返回的网卡信息数组及其内存
 * @param[in] network_cards 指向网卡信息数组的指针
 * @param[in] count 网卡信息数量
 * @note 调用此接口后，传入的 `network_cards` 数组及其中的每个元素将不再有效。
 */
void fdk_hwinfo_free_network_card_info(FDKNetworkCardInfo **network_cards, size_t count);

/**
 * 获取显示器信息。
 *
 * 该函数通过调用 `hwinfo --monitor` 获取系统中的显示器信息，并将其存储到一个动态分配的显示器数组中。
 * 返回的数组内存由函数内部分配，调用者在使用完后需调用 `fdk_hwinfo_free_display_info` 释放内存。
 * @param[out] count 返回显示器数量。成功时，`count` 将被设置为显示器的数量。
 * @param[out] errcode 错误码。成功时返回 0，失败时返回相应的错误码。
 * @return 成功返回指向显示器信息数组的指针，失败时返回 `NULL`。
 * @note 返回的数组和其中的每个显示器信息结构体的内存是动态分配的，调用者需要在使用完后调用 `fdk_hwinfo_free_display_info` 进行释放。
 * @warning 请确保调用完后释放返回的内存，以避免内存泄漏。
 */
FDKDisplayInfo **fdk_hwinfo_get_display_info(size_t *count, int *errcode);

/**
 * 释放由 `fdk_hwinfo_get_display_info` 返回的显示器信息数组及其内存
 * @param[in] displays 指向显示器信息数组的指针
 * @param[in] count 显示器信息数量
 * @note 调用此接口后，传入的 `displays` 数组及其中的每个元素将不再有效。
 */
void fdk_hwinfo_free_display_info(FDKDisplayInfo **displays, size_t count);

/**
 * 获取蓝牙设备信息，包含设备名称、MAC地址、信号强度、配对状态、连接状态等。
 *
 * 该函数通过系统命令获取当前蓝牙设备的相关信息，并将其存储到一个动态分配的蓝牙设备信息数组中。
 * 该函数会返回系统中所有蓝牙设备的信息，内存由函数内部分配，调用者在使用完返回的数组后需调用
 * `fdk_hwinfo_free_bluetooth_device_info` 释放内存。
 * @param[out] count 返回设备信息数量。
 * @param[out] errcode 错误码，成功时为 0，失败时键错误码。
 * @return 返回指向蓝牙设备信息数组的指针，失败时返回 NULL。
 */
FDKBluetoothDeviceInfo **fdk_hwinfo_get_bluetooth_adapter_info(size_t *count, int *errcode);
#if 0
/**
 * 释放蓝牙设备信息的内存。
 *
 * 该函数会释放由 `fdk_hwinfo_get_bluetooth_adapter_info` 函数返回的所有蓝牙设备信息。
 * 内存会逐个设备进行释放，最后释放设备信息数组本身的内存。
 * @param[in] info 指向蓝牙设备信息数组的指针。
 * @param[in] count 蓝牙设备的数量。
 */
void fdk_hwinfo_free_bluetooth_device_info(FDKBluetoothDeviceInfo **info, size_t count);
#endif
/**
 * 获取PCI设备信息，包含设备ID、设备名称等
 *
 * 该函数通过调用 `lspci` 命令并解析输出，获取当前系统中所有的PCI设备的相关信息。
 * 信息会存储在动态分配的结构体数组中。
 * @param[out] count 返回设备信息数量。
 * @param[out] errcode 错误码，成功时为 0，失败时为错误码。
 * @return 返回指向 `FDKPciDeviceInfo` 结构体数组的指针，失败时返回 NULL。
 */
FDKPciDeviceInfo **fdk_hwinfo_get_pci_device_info(size_t *count, int *errcode);

/**
 * 释放PCI设备信息的内存
 *
 * 该函数会释放由 `fdk_hwinfo_get_pci_device_info` 返回的所有PCI设备信息内存。
 * 内存会逐个设备进行释放，最后释放设备信息数组本身的内存。
 * @param[in] info 指向PCI设备信息数组的指针。
 * @param[in] count PCI设备的数量。
 */
void fdk_hwinfo_free_pci_device_info(FDKPciDeviceInfo **info, size_t count);


#ifdef __cplusplus
}
#endif

#endif // LIBFDHWINFO_H
