/**
 * Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
 * 
 * You should have find a copy of NFSChina License. If not, please contact us 
 * by email <os_support@nfschina.com>  
 * 
**/

#ifndef _LIBFD_NETWORK_H
#define _LIBFD_NETWORK_H
#ifdef __cplusplus
extern "C"
{
#endif

#include <stdbool.h>

typedef struct{
        unsigned int index;				     /**< 网络接口索引 */
        unsigned char *device_name;         /**< 网络设备名称 */
        unsigned char *mac_address;         /**< MAC 地址，格式为 XX:XX:XX:XX:XX:XX */
        unsigned char *ipv4_address;        /**< IP 地址，IPv4 格式（例如 192.168.1.1） */
        unsigned char *ipv6_address;        /**< IP 地址，IPv6 格式 */
        unsigned char *netmask;             /**< 子网掩码，IPv4 格式（例如 255.255.255.0） */
        unsigned char *broadaddr;           /**<  广播地址 */
        long mtu;                           /**< 最大传输单元（Maximum Transmission Unit） */
        int is_up;                          /**< 设备是否启用（网络设备是否处于活动状态） */
        bool is_wireless;                   /**< 是否是无线网络设备（例如 WiFi） */
} FDKNetworkDeviceInfo;


typedef struct {
        unsigned char *ip_address;           /**< 当前接口的 IP 地址 */
        unsigned char *subnet_mask;          /**< 子网掩码 */
        unsigned char *broadcast;            /**< 广播地址 */
} FDKSubnetInfo;

/**
   * 获取所有网络设备信息
   * @param[out] count 描述网络设备的数量
   * @return 返回指向网络设备信息FDKNetworkDeviceInfo的指针数组,失败时返回NULL
   * @warning 无
   * @note 需要调用fdk_netinfo_all_network_devices_free进行内存释放
   * @see 无
   */
    FDKNetworkDeviceInfo **fdk_netinfo_get_all_network_devices(int *count);

     /**
   * 释放fdk_netinfo_get_all_network_devices接口内部申请的内存空间
   * @param[in] devinfos 指向网络设备信息FDKNetworkDeviceInfo的指针数组
   * @param[in] count 描述网络设备的数量
   * @return 错误码
   * @warning 无
   * @note 无
   * @see 无
   */
    int fdk_netinfo_all_network_devices_free(FDKNetworkDeviceInfo **devinfos, int count);

/**
   * 启用指定的网络设备
   * @param[in] device_name 网络设备名称
   * @return 成功时返回0, 其他表示失败
   * @warning 无
   * @note 调用该接口需要root权限
   * @see 无
   */
    int fdk_netinfo_enable_network_device(const unsigned char *device_name);

/**
   * 禁用指定的网络设备
   * @param[in] device_name 网络设备名称
   * @return 成功时返回0, 其他表示失败
   * @warning 无
   * @note 调用该接口需要root权限
   * @see 无
   */
    int fdk_netinfo_disable_network_device(const unsigned char *device_name);

/**
   * 设置网络连接
   * @param[in] device_name 要设置的设备名称（如 eth0、wlan0）
   * @param[in] ip_address 要设置的IP地址（IPv4格式，如 192.168.1.100）
   * @param[in] netmask 要设置的子网掩码（如 255.255.255.0）
   * @return 成功返回0,否则返回错误码
   * @warning 无
   * @note 调用该接口需要root权限
   * @see 无
   */
    int fdk_netinfo_setting_by_name(const unsigned char *device_name,
                                    const unsigned char *ip_address,
                                    const unsigned char *netmask);

/**
   * 设置网络DNS
   * @param[in]  dns_servers 要设置的DNS服务器地址（如 8.8.8.8）
   * @return 成功返回0,否则返回错误码
   * @warning 无
   * @note 调用该接口需要root权限
   * @see 无
   */
    int fdk_netinfo_set_dns(const unsigned char *dns);


/**
   * 设置网络连接断开
   * @param[in]  device_name 需要断开的网卡名称
   * @return 成功返回0,否则返回错误码
   * @warning 无
   * @note 
   * @see 无
   */
    int fdk_netinfo_disconnect_network(const unsigned char *device_name);

/**
   * 设置网络连接
   * @param[in]  device_name 需要断开的网卡名称
   * @return 成功返回0,否则返回错误码
   * @warning 无
   * @note 
   * @see 无
   */
    int fdk_netinfo_connect_network(const unsigned char *device_name);

/**
   * 查询当前防火墙是否开启
   * @param 无
   * @return 1 表示防火墙已开启;  0 表示防火墙未开启; -1 表示查询失败
   * @warning 无
   * @note 
   * @see 需要用root权限执行
   */
    int fdk_netinfo_is_firewall_enabled(void);

/**
   * 通过网卡名获取当前所有网络接口的信息
   * @param [out] subnet_info 指向网络接口信息的指针
   * @param [in] device_name 指向网络接口名字的指针
   * @return 成功返回0， 
   * @warning 无
   * @note 
   * @see 
   */
    int fdk_netinfo_get_subnet_info(FDKSubnetInfo **subnet_info, unsigned char* device_name);

/**
   * 释放通过fdk_netinfo_get_subnet_info分配的空间
   * @param [out] subnet_info 指向网络接口信息的指针
   * @return 成功返回0， 
   * @warning 无
   * @note 
   * @see 
   */
    int fdk_netinfo_subnet_free(FDKSubnetInfo **subnet_info);

/**
   * 通过IP获取/etc/hosts映射的主机名
   * @param [in] ip ip地址的字符串
   * @param [out] hostname 指向存放主机名的指针
   * @return 成功返回0， 
   * @warning 无
   * @note 
   * @see 
   */
int fdk_netinfo_get_hostname_by_IP(const unsigned char* ip, unsigned char **hostname);

/**
   * 通过主机名获取/etc/hosts映射的ip
   * @param [in] hostname 主机名的字符串
   * @param [out] ip 指向存放ip的指针
   * @return 成功返回0， 
   * @warning 无
   * @note 
   * @see 
   */
int fdk_netinfo_get_IP_by_hostname(const unsigned char* hostname, unsigned char **ip);

/**
   * 获取当前 DNS 
   * @param [out] count dns的数量
   * @return 指向dns字符串的指针数组 
   * @warning 无
   * @note 
   * @see 
   */
unsigned char ** fdk_netinfo_get_dns_config(int *count);

/**
   * 获取当前系统的默认网关地址 
   * @param [out] gateway dns的数量
   * @return 指向gateway字符串的指针数组 
   * @warning 无
   * @note 
   * @see 
   */
int fdk_netinfo_get_default_gateway(unsigned char **gateway);

/**
   * 获取当前系统的默认网关地址 
   * @param [out] route dns的数量
   * @return 指向route字符串的指针数组 
   * @warning 无
   * @note 
   * @see 
   */
int fdk_netinfo_get_default_route(unsigned char **route);

#if 0
/**
   * 获取无线设备的工作模式 
   * @param [in] device_name 无线网卡的名称
   * @param [out] mode 无线设备的工作模式
   * @return 成功返回0,  
   * @warning 无
   * @note 
   * @see 
   */
int fdk_netinfo_get_wireless_mode(const unsigned char *device_name, unsigned char **mode);
#endif
#if 0
/**
   * 获取无线设备的工作频率 
   * @param [in] device_name 无线网卡的名称
   * @param [out] frequency 无线设备的工作频率
   * @return 成功返回0,  
   * @warning 无
   * @note 
   * @see 
   */
int fdk_netinfo_get_wireless_frequency(const unsigned char *device_name, unsigned char **frequency);
#endif
#if 0
/**
   * 获取无线设备的工作频道 
   * @param [in] device_name 无线网卡的名称
   * @param [out] channels 无线设备的工作频道
   * @return 成功返回0,  
   * @warning 无
   * @note 
   * @see 
   */
int fdk_netinfo_get_wireless_channels(const unsigned char *device_name, unsigned char **channels);
#endif
#if 0
/**
   * 获取无线设备的工作速率
   * @param [in] device_name 无线网卡的名称
   * @param [out] rate 无线设备的工作速率
   * @return 成功返回0,  
   * @warning 无
   * @note 
   * @see 
   */
int fdk_netinfo_get_wireless_rate(const unsigned char *device_name, unsigned char **rate);
#endif
#if 0
/**
   * 获取无线设备的接收灵敏度下限
   * @param [in] device_name 无线网卡的名称
   * @param [out] rate 无线设备的接收灵敏度下限
   * @return 成功返回0,  
   * @warning 无
   * @note 
   * @see 
   */
int fdk_netinfo_get_wireless_signal_level(const unsigned char *device_name, unsigned char **signal_level);
#endif

#ifdef __cplusplus
}
#endif
#endif
