/**
 * Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
 * 
 * You should have find a copy of NFSChina License. If not, please contact us 
 * by email <os_support@nfschina.com>  
 * 
**/

#ifndef LIBFDPOWER_H
#define LIBFDPOWER_H

#ifdef __cplusplus
extern "C" {
#endif

/*电源计划类型，枚举定义了不同的电源计划模式，包括节能模式、平衡模式和高性能模式。*/
typedef enum {
    POWER_SAVE = 0,    /**< 节能模式，优先节省电力，适合笔记本等电池设备 */
    POWER_BALANCE,     /**< 平衡模式，电力和性能之间的平衡，适用于日常使用 */
    POWER_PERFORMANCE  /**< 高性能模式，优先提升系统性能，适合高负载任务 */
} FDKPowerPlan;

/*电源选项类型，枚举定义了不同的电源选项类型，包括使用电源、使用电池。*/
typedef enum {
    POWER_AC = 0,    /**< 使用电源 */
    POWER_BATTERY     /**< 使用电池 */
} FDKPowerType;

//系统支持的休眠方式
typedef enum {
    SLEEP_MODE_FREEZE,    /**< 冻结模式 */
    SLEEP_MODE_DISK,      /**< 休眠到磁盘 */
    SLEEP_MODE_STANDBY,   /**< 待机模式 */
    SLEEP_MODE_MEM,       /**< 内存挂起 */
    SLEEP_MODE_UNKNOWN    /**< 未知模式 */
} FDKSleepMode;

/**
 * 获取无操作则关闭屏幕的时间。
 * @param[out] second 无操作则关闭屏幕的时间,单位：秒;0代表永不
 * @param[in] type POWER_AC代表使用电源，POWER_BATTERY代表使用电池
 * @return 成功返回0，否则返回错误码
 * @warning 无
 * @note 无
 * @see 无
 */
int fdk_pw_get_inactivity_turn_off_screen_time(int *second,FDKPowerType type);

/**
 * 获取无操作则睡眠的时间。
 * @param[out] second 无操作则睡眠的时间。,单位：秒;0代表永不
 * @param[in] type POWER_AC代表使用电源，POWER_BATTERY代表使用电池
 * @return 成功返回0，否则返回错误码
 * @warning 无
 * @note 无
 * @see 无
 */
int fdk_pw_get_inactivity_sleep_time(int *second,FDKPowerType type);

/**
 * 获取设备支持的休眠方式
 * @param[out] sleep_modes 存放设备支持的休眠方式的枚举数组，内存由 API 内部分配
 * @param[out] count 返回支持的休眠方式数量
 * @return 成功返回0，否则返回错误码
 * @warning 调用者需要确保调用完后释放 `sleep_modes` 所指向的内存
 * @note 无
 * @see 无
 */
int fdk_pw_get_supported_sleep_modes(FDKSleepMode **sleep_modes, int *count);

/**
 * 检测当前设备的电源模式
 * @param[out] plan 指向FDKPowerPlan枚举类型的指针，用于存储当前的电源计划
 * @return 成功返回0，否则返回错误码
 * @warning 无
 * @note 无
 * @see 无
 */
int fdk_pw_get_current_power_plan(FDKPowerPlan *plan);

/**
 * 获取当前设备的电池电量。
 * @param[out] precent 电池电量百分比，范围为 0 到 100
 * @return 成功返回0，否则返回错误码
 * @warning 无
 * @note 无
 * @see 无
 */
int fdk_pw_get_battery_level(int *precent);

#ifdef __cplusplus
}
#endif

#endif // FDPOWER_H
