/**
 * Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
 * 
 * You should have find a copy of NFSChina License. If not, please contact us 
 * by email <os_support@nfschina.com>  
 * 
**/

#ifndef LIBFDPRINTER_H
#define LIBFDPRINTER_H

#include <time.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
    PRINTER_STATUS_IDLE = 3,       /* 打印机空闲 */
    PRINTER_STATUS_PROCESSING,     /* 打印机工作中 */
    PRINTER_STATUS_STOPPED,        /* 打印机停止 */
    PRINTER_STATUS_UNKNOWN         /* 打印机状态未知 */
} FDKPrinterStatus;

typedef enum 		/**** Job states ****/
{
  IPP_JOBSTATUS_PENDING = 3,		/* Job is waiting to be printed */
  IPP_JOBSTATUS_HELD,			/* Job is held for printing */
  IPP_JOBSTATUS_PROCESSING,		/* Job is currently printing */
  IPP_JOBSTATUS_STOPPED,			/* Job has been stopped */
  IPP_JOBSTATUS_CANCELED,			/* Job has been canceled */
  IPP_JOBSTATUS_ABORTED,			/* Job has aborted due to error */
  IPP_JOBSTATUS_COMPLETED,		/* Job has completed successfully */
  IPP_JOBSTATUS_UNKNOWN
} FDKipp_jstate_t;

typedef struct 		/**** Printer Options ****/
{
  char		*name;			/* Name of option */
  char		*value;			/* Value of option */
} FDKcups_option_t;

typedef struct 		/**** Destination ****/
{
  char		*name,			/* Printer or class name */
        *instance;		/* Local instance name or NULL */
  int		is_default;		/* Is this printer the default? */
  int		num_options;		/* Number of options */
  FDKcups_option_t	*options;		/* Options */
} FDKCupsDest;

typedef struct 		/**** Job ****/
{
  int		id;			/* The job ID */
  char		*dest;			/* Printer or class name */
  char		*title;			/* Title/job name */
  char		*user;			/* User that submitted the job */
  char		*format;		/* Document format */
  FDKipp_jstate_t	state;			/* Job state */
  int		size;			/* Size in kilobytes */
  int		priority;		/* Priority (1-100) */
  time_t	completed_time;		/* Time the job was completed */
  time_t	creation_time;		/* Time the job was created */
  time_t	processing_time;	/* Time the job was processed */
} FDKcups_job_t;

/**
 * 获取打印机列表
 * @param[out] dests 将指向FDKCupsDest结构体数组缓冲区指针的地址作为参数，使用后需通过cupsFreeDests释放
 * @return 成功返回0，否则返回错误码
 * @warning 调用完后使用fdk_cups_free_cups_dests释放内存
 * @note 无
 * @see 无
 */
int fdk_cups_get_cups_dests(FDKCupsDest **dests, int *count);

/**
 * 释放由 `fdk_cups_get_cups_dests` 获取的打印机列表内存
 * @param[in] dests dests 指向 `FDKCupsDest` 结构体数组的指针
 * @param[in] count 打印机数量
 * @return 无
 * @warning 无
 * @note 无
 * @see 无
 */
void fdk_cups_free_cups_dests(FDKCupsDest *dests, int count);

/**
 * 设置打印参数（在打印前使用）
 * @param[out] options 将指向cups_option_t结构体数组缓冲区指针的地址作为参数
 * @param[in] name 打印参数名，例："PageSize"
 * @param[in] value 打印参数值，例如："A4"
 * @return 成功返回0，否则返回错误码
 * @warning 确保传入的name、value有效
 * @note 无
 * @see 无
 */
int fdk_cups_set_printer_options(const char *name,const char *value,FDKcups_option_t **options);

/**
 * 打印文件
 * @param[in] job_id 任务id
 * @param[in] name 打印机名称
 * @param[in] filename 打印文件绝对路径
 * @param[in] title 打印任务名
 * @param[in] num_options 打印选项的数量
 * @param[in] options 打印选项，将指向cups_option_t结构体数组缓冲区指针的地址作为参数，使用后通过cupsFreeOptions释放
 * @return 成功返回0，否则返回错误码
 * @warning 确保传入的name、filename、title有效
 * @note 无
 * @see 无
 */
int fdk_cups_print_file(const char *name, const char *filename,const char *title, int num_options,FDKcups_option_t **options,int *job_id);

/**
 * 取消打印
 * @param[in] name 打印机名称
 * @param[in] job_id 任务id
 * @return 成功返回0，否则返回错误码
 * @warning 确保传入的name、job_id有效
 * @note 无
 * @see 无
 */
int fdk_cups_cancel_print_job(const char *name, int job_id);

/**
 * 获取指定打印机的当前状态。
 * @param[in] printer_name 打印机的名称
 * @param[out] 打印机的状态
 * @return 成功返回0，否则返回错误码
 * @warning 确保传入的 `printer_name` 为有效的打印机名称。
 * @note 无
 * @see 无
 */
int fdk_cups_get_printer_status(const char *printer_name, FDKPrinterStatus* status);

/**
 * 获取指定打印任务的当前状态。
 * @param[in]  job_id 打印任务的 ID
 * @param[out] 打印任务的状态
 * @return 成功返回0，否则返回错误码
 * @warning 确保传入的 `job_id` 为有效的打印任务ID。
 * @note 无
 * @see 无
 */
int  fdk_cups_get_print_job_status(int job_id, FDKipp_jstate_t *status);

#ifdef __cplusplus
}
#endif


#endif // FDPRINTER_H
