/**
 * Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
 * 
 * You should have find a copy of NFSChina License. If not, please contact us 
 * by email <os_support@nfschina.com>  
 * 
**/

#ifndef LIBFDSYSINFO_H
#define LIBFDSYSINFO_H

#include <pwd.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @struct FDKUserAccount
 * @brief 用户账户信息
 * 
 * 该结构体存储有关用户账户的详细信息，包括用户名、用户ID、所属组ID、家目录和登录Shell等。
 */
typedef struct {
    unsigned char *username;                   /*< 用户名 */
    uid_t user_id;                                       /*< 用户ID */
    gid_t group_id;                                    /*< 用户所属组ID */
    unsigned char *home_directory;         /*< 用户的家目录 */
    unsigned char *shell;                           /*< 用户的登录 shell */
} FDKUserAccount;

/**
 * 获取操作系统名称。
 *
 * 该接口从系统信息文件中提取操作系统的名称，并将结果返回给调用者。
 * 内存由函数内部分配，调用者在使用完后需调用 free() 释放内存，以避免内存泄漏。
 *
 * @param[out] os_name 指向操作系统名称的指针地址。函数内部分配内存，并将名称字符串赋值给该指针。
 *                     调用者负责释放返回的内存。
 * @return int 成功返回 0，失败返回错误码。 *
 * @note 返回的字符串为动态分配的内存。调用者需在使用完后释放该内存。
 * @warning 确保传入的指针非空，并正确释放返回的内存，以避免内存泄漏。
 * @see 无
 */
int fdk_sysinfo_get_os_name(unsigned char **os_name);

/**
 * 获取操作系统名称（英文）。
 *
 * 该接口从系统信息文件中提取操作系统的英文名称，并将结果返回给调用者。
 * 内存由函数内部分配，调用者在使用完后需调用 free() 释放内存，以避免内存泄漏。
 *
 * @param[out] os_name 指向操作系统英文名称的指针地址。函数内部分配内存，并将名称字符串赋值给该指针。
 *                     调用者负责释放返回的内存。
 * @return int 成功返回 0，失败返回错误码。
 * @note 返回的字符串为动态分配的内存。调用者需在使用完后释放该内存。
 * @warning 确保传入的指针非空，并正确释放返回的内存，以避免内存泄漏。
 * @see 无
 */
int fdk_sysinfo_get_os_name_en(unsigned char **os_name);

/**
 * 获取系统版本信息。
 *
 * 该接口从系统信息文件中提取系统版本号，并将结果返回给调用者。
 * 内存由函数内部分配，调用者在使用完后需调用 free() 释放内存，以避免内存泄漏。
 *
 * @param[out] version 指向系统版本信息的指针地址。函数内部分配内存，并将版本号字符串赋值给该指针。
 *                     调用者负责释放返回的内存。
 * @return int 成功返回 0，失败返回错误码。
 * @note 返回的字符串为动态分配的内存。调用者需在使用完后释放该内存。
 * @warning 确保传入的指针非空，并正确释放返回的内存，以避免内存泄漏。
 * @see 无
 */
int fdk_sysinfo_get_os_version(unsigned char **version);

/**
 * 获取系统补丁版本号。
 *
 * 该接口从系统信息文件中提取系统补丁版本号，并将结果返回给调用者。
 * 内存由函数内部分配，调用者在使用完后需调用 free() 释放内存，以避免内存泄漏。
 *
 * @param[out] version 指向系统补丁版本信息的指针地址。函数内部分配内存，并将补丁版本号字符串赋值给该指针。
 *                     调用者负责释放返回的内存。
 * @return int 成功返回 0，失败返回错误码。
 * @note 返回的字符串为动态分配的内存。调用者需在使用完后释放该内存。
 * @warning 确保传入的指针非空，并正确释放返回的内存，以避免内存泄漏。
 * @see 无
 */
int fdk_sysinfo_get_os_patch_version(unsigned char **version);

/**
 * 获取系统内核版本信息。
 *
 * 该接口从系统信息文件中提取系统内核版本号，并将结果返回给调用者。
 * 内存由函数内部分配，调用者在使用完后需调用 free() 释放内存，以避免内存泄漏。
 *
 * @param[out] version 指向内核版本信息的指针地址。函数内部分配内存，并将内核版本号字符串赋值给该指针。
 *                     调用者负责释放返回的内存。
 * @return int 成功返回 0，失败返回错误码。
 * @note 返回的字符串为动态分配的内存。调用者需在使用完后释放该内存。
 * @warning 确保传入的指针非空，并正确释放返回的内存，以避免内存泄漏。
 * @see 无
 */
int fdk_sysinfo_get_kernel_version(unsigned char **version);

/**
 * 获取系统构建时间。
 *
 * 该接口从系统信息文件中提取系统构建时间，并将结果返回给调用者。
 * 内存由函数内部分配，调用者在使用完后需调用 free() 释放内存，以避免内存泄漏。
 *
 * @param[out] build_time 指向系统构建时间字符串的指针地址。函数内部分配内存，并将系统构建时间字符串赋值给该指针。
 *                        调用者负责释放返回的内存。
 * @return int 成功返回 0，失败返回错误码。
 * @note 返回的字符串为动态分配的内存。调用者需在使用完后释放该内存。
 * @warning 确保传入的指针非空，并正确释放返回的内存，以避免内存泄漏。
 * @see 无
 */
int fdk_sysinfo_get_os_build_time(unsigned char **build_time);

/**
 * 获取主机名。
 *
 * 该接口获取系统的主机名，并将结果返回给调用者。
 * 内存由函数内部分配，调用者在使用完后需调用 free() 释放内存，以避免内存泄漏。
 *
 * @param[out] host_name 指向主机名字符串的指针地址。函数内部分配内存，并将主机名字符串赋值给该指针。
 *                      调用者负责释放返回的内存。
 * @return int 成功返回 0，失败返回错误码。
 * @note 返回的字符串为动态分配的内存。调用者需在使用完后释放该内存。
 * @warning 确保传入的指针非空，并正确释放返回的内存，以避免内存泄漏。
 * @see 无
 */
int fdk_sysinfo_get_host_name(unsigned char **host_name);

/**
 * 获取系统架构信息。
 *
 * 该接口根据系统预定义的宏获取系统的架构信息，并将结果返回给调用者。
 * 内存由函数内部分配，调用者在使用完后需调用 free() 释放内存，以避免内存泄漏。
 *
 * @param[out] architecture 指向系统架构信息字符串的指针地址。函数内部分配内存，并将架构信息字符串赋值给该指针。
 *                          调用者负责释放返回的内存。
 * @return int 成功返回 0，失败返回错误码。
 * @note 返回的字符串为动态分配的内存。调用者需在使用完后释放该内存。
 * @warning 确保传入的指针非空，并正确释放返回的内存，以避免内存泄漏。
 * @see 无
 */
int fdk_sysinfo_get_architecture(unsigned char **architecture);

/**
 * 获取系统激活状态
 * @return 0: 试用期(未激活); 1: 已激活; 2: 激活(已过期); 3: 试用期; 4: 试用期(已过期); -1: 失败。
 * @note 使用 D-Bus 接口获取许可证状态。
 */
int fdk_sysinfo_get_active_state(void);

/**
 * 获取操作系统序列号。
 *
 * 该接口从系统文件中读取操作系统的序列号，并将其返回给调用者。
 * 内存由函数内部分配，调用者需手动释放内存以避免内存泄漏。
 *
 * @param[out] sn 指向操作系统序列号的指针地址。函数内部分配内存，并将序列号字符串赋值给该指针。
 *                调用者负责释放返回的内存。
 * @return int 成功返回 0，失败返回错误码。
 * @warning 调用者需确保释放返回的内存，避免内存泄漏。
 * @note 返回的序列号是操作系统的唯一标识。
 * @see 无
 */
int fdk_sysinfo_get_system_sn(unsigned char **sn);

/**
 * 获取整机制造商。
 *
 * 该接口从系统文件中读取整机制造商名称并返回。
 * 内存由函数内部分配，调用者需手动释放内存以避免内存泄漏。
 *
 * @param[out] manufacturer 指向制造商名称的指针地址。函数内部分配内存，并将字符串赋值给该指针。
 *                          调用者负责释放返回的内存。
 * @return int 成功返回 0，失败返回错误码。
 * @warning 调用者需确保释放返回的内存，避免内存泄漏。
 * @note 返回计算机制造商的名称。
 * @see 无
 */
int fdk_sysinfo_get_manufacturer(unsigned char **manufacturer);

/**
 * 获取整机型号。
 *
 * 该接口从系统文件中读取整机型号名称并返回。
 * 内存由函数内部分配，调用者需手动释放内存以避免内存泄漏。
 *
 * @param[out] model 指向型号名称的指针地址。函数内部分配内存，并将字符串赋值给该指针。
 *                   调用者负责释放返回的内存。
 * @return int 成功返回 0，失败返回错误码。
 * @warning 调用者需确保释放返回的内存，避免内存泄漏。
 * @note 返回计算机的型号名称。
 * @see 无
 */
int fdk_sysinfo_get_machine_model(unsigned char **model);

/**
 * 获取整机序列号。
 *
 * 该接口从系统文件中读取计算机的唯一序列号并返回。
 * 内存由函数内部分配，调用者需手动释放内存以避免内存泄漏。
 *
 * @param[out] serial 指向序列号的指针地址。函数内部分配内存，并将字符串赋值给该指针。
 *                    调用者负责释放返回的内存。
 * @return int 成功返回 0，失败返回错误码。
 * @warning 调用者需确保释放返回的内存，避免内存泄漏。
 * @note 返回计算机的唯一序列号。
 * @see 无
 */
int fdk_sysinfo_get_machine_serial(unsigned char **serial);

/**
 * 获取当前登录用户账号。
 *
 * 该接口从系统获取当前登录用户的账户信息，包括用户名、用户ID、组ID、主目录、shell等。
 * 内存由函数内部分配，调用者需手动释放内存以避免内存泄漏。
 *
 * @param[out] account 指向 `FDKUserAccount` 结构体的指针地址。函数内部分配内存，并将账户信息赋值给该指针。
 *                     调用者负责释放返回的内存。
 * @return int 成功返回 0，失败返回错误码。
 * @warning 调用者需确保释放返回的内存，避免内存泄漏。
 * @note 返回当前用户的账户信息，包括用户名、用户ID、组ID、主目录、shell等。
 * @see fdk_sysinfo_free_user_account
 */
int fdk_sysinfo_get_current_account(FDKUserAccount **account);

/**
 * 释放由 fdk_sysinfo_get_current_account 返回的内存
 * @param account 指向需要释放的用户账户信息结构体指针
 * @note 调用此接口后，传入的 account 指针将不再有效。
 */
void fdk_sysinfo_free_user_account(FDKUserAccount *account);

/**
 * 获取系统所有用户账号。
 *
 * 该接口用于获取系统中所有用户的账户信息，返回一个指向 `FDKUserAccount` 结构体数组的指针。
 * 调用者需负责释放返回的用户数组和每个用户的内存，以避免内存泄漏。
 *
 * @param[out] count 返回用户账户数量。
 * @param[out] errcode 错误代码，成功时为 0，失败时见错误码。
 * @return 返回指向 `FDKUserAccount` 结构体数组的指针，成功时返回数组，失败时返回 `NULL`。
 */
FDKUserAccount **fdk_sysinfo_get_all_users(size_t *count, int *errcode);

/**
 * 释放由 fdk_sysinfo_get_all_users 返回的用户账户信息数组及其内存
 * @param[in] users 指向用户账户信息数组的指针
 * @param[in] count 用户数量
 * @note 调用此接口后，传入的用户数组和其中的每个元素将不再有效。
 */
void fdk_sysinfo_free_all_users(FDKUserAccount **users, size_t count);

/**
 * 获取系统安装时间。
 *
 * 该接口从系统安装时间文件中读取并返回系统的安装时间，时间格式为 "YYYY-MM-DD HH:MM:SS"。
 * 调用者需负责释放返回的字符串，以避免内存泄漏。
 *
 * @param[out] install_time 返回指向系统安装时间的指针。
 *                          调用者在使用完后需调用 free() 释放返回的内存。
 * @return int 成功返回 0，失败返回错误码。
 * @warning 调用者需确保释放返回的时间字符串，避免内存泄漏。
 * @note 返回的时间格式为 "YYYY-MM-DD HH:MM:SS"。
 */
int fdk_sysinfo_get_system_install_time(unsigned char **install_time);

/**
 * 获取系统开机时间。
 *
 * 该接口从系统文件 `/proc/stat` 获取系统的开机时间，并将其格式化为 "YYYY-MM-DD HH:MM:SS"。
 * 调用者需负责释放返回的字符串，以避免内存泄漏。
 *
 * @param[out] boot_time 返回指向系统开机时间的指针。
 *                       调用者在使用完后需调用 free() 释放返回的内存。
 * @return int 成功返回 0，失败返回错误码。
 * @warning 调用者需确保释放返回的时间字符串，避免内存泄漏。
 * @note 返回的时间格式为 "YYYY-MM-DD HH:MM:SS"。
 */
int fdk_sysinfo_get_system_boot_time(unsigned char **boot_time);

/**
 * 获取系统最后一次关机时间
 * @param[out] last_shutdown_time 返回指向关机时间的指针。
 * @return 成功返回 0，失败返回错误码。
 * @warning 调用者需确保释放返回的字符串，避免内存泄漏。
 * @note 返回的时间格式为 "YYYY-MM-DD HH:MM:SS"。
 */
int fdk_sysinfo_get_last_shutdown_time(unsigned char **last_shutdown_time);

/**
 * 获取系统当前时区
 * @param[out] timezone 返回指向时区字符串的指针。
 * @return 成功返回 0，失败返回错误码。
 * @warning 调用者需确保释放返回的字符串，避免内存泄漏。
 * @note 获取系统时区信息，如果环境变量 `TZ` 设置了时区，则返回其值，否则返回 `"UTC"`。
 */
int fdk_sysinfo_get_system_timezone(unsigned char **timezone);

/**
 * 获取系统当前时间
 * @param[out] time_str 返回指向当前系统时间的指针。
 * @return 成功返回 0，失败返回错误码。
 * @warning 调用者需确保释放返回的字符串，避免内存泄漏。
 * @note 返回当前系统的时间，格式为 "YYYY-MM-DD HH:MM:SS"。
 */
int fdk_sysinfo_get_system_time(unsigned char **time_str);

/**
 * 获取系统中总进程数
 * @return 返回系统中的总进程数，如果获取失败返回 0。
 * @note 该函数读取 `/proc/stat` 文件来获取进程数量。
 */
unsigned long fdk_sysinfo_get_process_count();

/**
 * 获取进程默认栈大小
 * @return 返回进程默认栈大小。如果获取失败，返回0。
 * @note 该函数获取当前进程的默认栈大小，通常是通过线程的默认栈大小来获取。
 */
size_t fdk_sysinfo_get_process_default_stack_size();

/**
 * 获取进程最大打开文件数
 * @return 返回进程最大打开文件数。如果获取失败，返回0。
 * @note 该函数通过 `getrlimit` 获取当前进程的最大打开文件数。
 */
unsigned long fdk_sysinfo_get_process_max_open_file_count();


#ifdef __cplusplus
}
#endif

#endif // LIBFDSYSINFO_H
