/**
 * Copyright (c) 2024-2025 National Fundamental Software of China Co., Ltd.
 * 
 * You should have find a copy of NFSChina License. If not, please contact us 
 * by email <os_support@nfschina.com>  
 * 
**/

#ifndef LIBFDUSBINFO
#define LIBFDUSBINFO

#include <stdint.h>

#ifdef __cplusplus
extern "C"
{
#endif

    /**
     * @enum FDKUsbSpeed
     * 该枚举用于描述设备协商连接速度
     */
    typedef enum
    {
        USB_SPEED_UNKNOWN = 0,   /**< 未知速度 */
        USB_SPEED_LOW = 1,       /**< 设备正在低速运行 (1.5 MBit/s) */
        USB_SPEED_FULL = 2,      /**< 设备正在全速运行 (12 MBit/s) */
        USB_SPEED_HIGH = 3,      /**< 设备正在高速运行 (480 MBit/s) */
        USB_SPEED_SUPER = 4,     /**< 设备正在超高速运行 (5000 MBit/s) */
        USB_SPEED_SUPER_PLUS = 5 /**< 设备正在超超速运行 (10000 MBit/s) */
    } FDKUsbSpeed;

    /**
     * @struct FDKUsbDevice
     * @brief USB设备详细信息结构体
     * 该结构体用于描述USB设备的详细信息
     */
    typedef struct
    {
        uint8_t bLength;            /**< 设备描述符的大小(字节) */
        uint8_t bDescriptorType;    /**< 设备描述符类型,在此上下文中为libusb_descriptor_type::LIBUSB_DT_DEVICE LIBUSB_DT_DEVICE */
        uint16_t bcdUSB;            /**< 以二进制编码的十进制表示的USB规范版本号,例如:0x0200表示USB2.0,0x0110表示USB1.1等 */
        uint8_t bDeviceClass;       /**< 设备的USB-IF类代码 */
        uint8_t bDeviceSubClass;    /**< 设备的USB-IF子类代码,由bDeviceClass限定 */
        uint8_t bDeviceProtocol;    /**< 设备的USB-IF协议代码,由bDeviceClass和bDeviceSubClass限定 */
        uint8_t bMaxPacketSize0;    /**< 端点0的最大数据包大小 */
        uint16_t idVendor;          /**< 设备的USB-IF供应商ID */
        uint16_t idProduct;         /**< 设备的USB-IF产品ID */
        uint16_t bcdDevice;         /**< 以二进制编码的十进制表示的设备版本号 */
        uint8_t iManufacturer;      /**< 描述制造商的字符串描述符索引 */
        uint8_t iProduct;           /**< 描述产品的字符串描述符的索引 */
        uint8_t iSerialNumber;      /**< 包含设备序列号的字符串描述符索引 */
        uint8_t bNumConfigurations; /**< 可能的配置数量 */
    } FDKUsbDevice;

    /**
     * 获取计算机所有USB设备接口列表
     * @param[out] devs 将指向FDKUsbDevice结构体数组缓冲区指针的地址作为参数
     * @param[in] number USB设备个数
     * @return 成功返回0，否则返回错误码
     * @warning 确保传入的devs有效
     * @note 无
     * @see 无
     */
    int libusb_start_operation();

    /**
     * 获取计算机所有USB设备接口列表
     * @param[out] devs 将指向FDKUsbDevice结构体数组缓冲区指针的地址作为参数
     * @param[in] number USB设备个数
     * @return 成功返回0，否则返回错误码
     * @warning 确保传入的devs有效
     * @note 无
     * @see 无
     */
    int libusb_end_operation();

    /**
     * 获取计算机所有USB设备接口列表
     * @param[out] devs 将指向FDKUsbDevice结构体数组缓冲区指针的地址作为参数
     * @param[in] number USB设备个数
     * @return 成功返回0，否则返回错误码
     * @warning 确保传入的devs有效
     * @note 无
     * @see 无
     */
    int fdk_usb_get_device_list(FDKUsbDevice **devs, int *number);

    /**
     * 获取计算机中所有USB存储设备信息
     * @param[out] devs 将指向FDKUsbDevice结构体数组缓冲区指针的地址作为参数
     * @param[in] number USB存储设备个数
     * @return 成功返回0，否则返回错误码
     * @warning 确保传入的devs有效
     * @note 无
     * @see 无
     */
    int fdk_usb_get_storage_device_list(FDKUsbDevice **devs, int *number);

    /**
     * 获取指定USB设备的总线地址和设备地址
     * @param[in] idVendor 供应商id
     * @param[in] idProduct 产品id
     * @param[out] bus_number 总线地址
     * @param[out] device_address 设备地址
     * @return 成功返回0，否则返回错误码
     * @warning 确保传入的idVendor、idProduct有效
     * @note 无
     * @see 无
     */
    int fdk_usb_get_device_address_by_id(unsigned short *idVendor, unsigned short *idProduct, unsigned int *bus_number, unsigned int *device_address);

    /**
     * 获取指定USB存储设备是否被挂载
     * @param[out] FDKUsbDevice_path 指向存放设备路径的指针
     * @param[in] is_mounted 是否被挂载
     * @return 成功返回0，否则返回错误码
     * @warning 确保传入的FDKUsbDevice_path有效
     * @note 无
     * @see 无
     */
    int fdk_usb_is_storage_device_mounted(const unsigned char *usb_device_path, int *is_mounted);

    /**
     * 获取指定USB设备的详细信息
     * @param[in] idVendor 供应商id
     * @param[in] idProduct 产品id
     * @param[out] data 指向存放详细信息的指针
     * @return 成功返回0，否则返回错误码
     * @warning 确保传入的idVendor、idProduct有效
     * @note 无
     * @see 无
     */
    int fdk_usb_get_device_data(unsigned short *idVendor, unsigned short *idProduct, FDKUsbDevice **data);

    /**
     * 获取指定USB设备的大类及子类信息
     * @param[in] idVendor 供应商id
     * @param[in] idProduct 产品id
     * @param[out] bDeviceClass 大类信息
     * @param[out] bDeviceSubClass 子类信息
     * @return 成功返回0，否则返回错误码
     * @warning 确保传入的idVendor、idProduct有效
     * @note 无
     * @see 无
     */
    int fdk_usb_get_device_class_data(unsigned short *idVendor, unsigned short *idProduct, unsigned short *bDeviceClass, unsigned short *bDeviceSubClass);

    /**
     * 获取指定USB设备的协议码
     * @param[in] idVendor 供应商id
     * @param[in] idProduct 产品id
     * @param[out] bDeviceProtocol 协议码
     * @return 成功返回0，否则返回错误码
     * @warning 确保传入的idVendor、idProduct有效
     * @note 无
     * @see 无
     */
    int fdk_usb_get_device_protocol(unsigned short *idVendor, unsigned short *idProduct, unsigned short *bDeviceProtocol);

    /**
     * 获取指定USB设备的协商连接速度
     * @param[in] idVendor 供应商id
     * @param[in] idProduct 产品id
     * @param[out] speed_type 设备速度类型
     * @return 成功返回0，否则返回错误码
     * @warning 确保传入的idVendor、idProduct有效
     * @note 无
     * @see 无
     */
    int fdk_usb_get_device_speed(unsigned short *idVendor, unsigned short *idProduct, FDKUsbSpeed *speed_type);

#ifdef __cplusplus
}

#endif

#endif // LIBFDUSBINFO
